/*
 * ClothTex v0.11
 *
 * POV-Ray 3.5 macros for Woven Cloth Texture
 * by Kedar Patil <kedar@patilkedar.com>
 *
 * There should be a file clothtex.pov with this file.  It contains a
 * sample scene for demonstration.
 *
 */


/* 
 * Macro MakeClothPigment()
 *
 * Creates a cloth pigment based on following parameters 
 *
 *   WeaveArray: Integer array indicating the weave of fabric. 0 means that
 *   thread running parallel to X axis is on top.  1 means that thread
 *   running parallel to Y axis is on top.
 *               
 *   XColorArray: Array of colors of threads running parallel to Y axis.
 *
 *   YColorArray: Array of colors of threads running parallel to X axis.
 *      
 *   XThreadWidth: Width of thread that runs parallel to Y axis.
 *
 *   YThreadWidth: Width of thread that runs parallel to X axis.
 *
 *   XGapWidth: Gap between the threads running in Y direction.
 *
 *   YGapWidth: Gap between the threads running in X direction.
 * 
 *  Dimensions of WeaveArray determine the number of threads in each
 *  direction.  Rows of the array are parallel to X-Axis. 
 * 
 *  Thread and gap widths are used for ratios.  So the absolute value does
 *  not matter.  The pigment is ultimately sized at 1 x 1 pov unit no
 *  matter how many threads run in each direction.  If you use different
 *  number of threads in both directions (i.e. WeaveArray is not a square
 *  matrix) then you may want to scale accordingly.  
 *
 *  Also, the pattern lies in X-Y plane.  So if you are not using
 *  uv_mapping, take care to rotate as needed.
 * 
 * Names of parameters are a pain in the ass.  So I think an explanation is
 * in order --
 *
 *  XColorArray runs in X direction but the colors are of threads running
 *  parallel to Y axis!  Similarly for YColorArray.
 *
 *  XThreadWidth is width in X direction.  YThreadWidth is width in Y
 *  direction.  Similarly XGapWidth is gap width in X direction and
 *  YGapWidth is gap width in Y direction.
 *
 * Was that easy?  Then please explain it to me ;-) 
 *
 * Was that difficult?  Well, you should be thankful to me that I didn't
 * name everything in terms of "warps" and "wefts"!  (Actually, I *had*
 * started doing it and then made a complete mess of everything in my mind)
 *
 * Whatever.  Enjoy the macros and make beautiful cloths!
 *
 * -Kedar
 *
 */


#macro MakeClothPigment (WeaveArray, XColorArray, YColorArray,
    XThreadWidth, YThreadWidth, XGapWidth, YGapWidth)

pigment
{
    #local TransColor = color rgbft 1;

    // number of threads in x and y directions
    #local nX = dimension_size(WeaveArray, 2);
    #local nY = dimension_size(WeaveArray, 1);

    // number of colors in x and y directions
    #local nCX = dimension_size(XColorArray, 1);
    #local nCY = dimension_size(YColorArray, 1);

    /*
     * Thread width in x and y directions as well as width of gaps in x and
     * y direction.  Normalized so that the cloth is 1 pov unit in each
     * direction.
     */
    
    #local total = XThreadWidth*nX + XGapWidth*nX;
    #local dX  = XThreadWidth/total;
    #local dGX = XGapWidth/total;

    #local total = YThreadWidth*nY + YGapWidth*nY;
    #local dY  = YThreadWidth/total;
    #local dGY = YGapWidth/total;


    #local i = 0;

    #local TempPigment = array[nY];

    #while(i < nY)
        
        #local start = 0;
        #local stop  = 0;
        #local YTempColor = YColorArray[mod(i,nCY)];

        #local TempPigment[i]=
        pigment
        {
            gradient x
            color_map
            {
        
                #local j = 0;
                #while(j < nX)
         
                    #local XTempColor = XColorArray[mod(j,nCX)];

                    // if the thread running across is below
                    #if(ClothWeaveArray[i][j] != 0)
                         
                        #local stop = stop + dX;

                        [ start XTempColor ]
                        [ stop  XTempColor ]

                        #local start = stop;

                        #local stop = stop + dGX;
                                
                        [ start YTempColor ]
                        [ stop  YTempColor ]

                        #local start = stop;
                                    
                    #else // thread running across is above
                        
                        #local stop = stop + dX + dGX;

                        [ start YTempColor ]
                        [ stop  YTempColor ]

                        #local start = stop;
                                    
                    #end

                    #local start = stop;
                    
                    #local j = j + 1;
                 
                #end

                #local i = i + 1;
                            
            }
        }

    #end

    
    #local TempPigmentGap=
    pigment
    {
        gradient x

        color_map
        {
            #local j = 0;
            #local start = 0;
            #local stop = 0;

            #local TempGapColor = Clear;
            #while(j < nX)
                #local TempColor = XColorArray[mod(j,nCX)];
                
                #local stop = stop + dX;
                [ start TempColor ]
                [ stop  TempColor ]

                #local start = stop;

                #local stop = stop + dGX;
                [ start TempGapColor ]
                [ stop  TempGapColor ]

                #local start = stop;
                
                #local j = j + 1;
            #end
        }
    }


    gradient y

    pigment_map
    {
        #local i = 0;
        #local start = 0;
        #local stop = 0;

        #while(i < nY)
            
            #local stop = stop + dY;
            [ start TempPigment[i] ]
            [ stop  TempPigment[i] ]
            #local start = stop;
        
            #local stop = stop + dGY;
            [ start TempPigmentGap ]
            [ stop  TempPigmentGap ]
            #local start = stop;
            
        #local i = i + 1;
        #end
    }
}

#end  // MakeClothPigment



/*
 * Macro MakeClothNormal()
 *
 * Creates cloth normal for use with ClothPigment.
 *
 * Parameters are same as in MakeClothPigment but the color arrays are not
 * needed.
 *
 * This is the most alpha code.  The normal looks nice from a distance but
 * when you want to see the weave closely, it falters.  Also the threads
 * are modeled as thin stripes instead of solid sphere-sweeplike shape.
 */

#macro MakeClothNormal (WeaveArray, XThreadWidth, YThreadWidth, XGapWidth,
    YGapWidth)
normal
{
    // number of threads in x and y directions
    #local nX = dimension_size(WeaveArray, 2);
    #local nY = dimension_size(WeaveArray, 1);

    /*
     * Thread width in x and y directions as well as width of gaps in x and
     * y direction.  Normalized so that the cloth is 1 pov unit in each
     * direction.
     */
    
    #local total = XThreadWidth*nX + XGapWidth*nX;
    #local dX  = XThreadWidth/total;
    #local dGX = XGapWidth/total;

    #local total = YThreadWidth*nY + YGapWidth*nY;
    #local dY  = YThreadWidth/total;
    #local dGY = YGapWidth/total;

    #local ClearNormal=
    normal
    {
        gradient x
        slope_map
        {
            [0.0 <0.5, 0.0>]
            [1.0 <0.5, 0.0>]
        }
    }

    /* 
     * Helper macros -- only for internal use
     */

    #macro _HighThread(_start, _len)
        [ _start+0.0*_len <1, 0> ]
        [ _start+0.2*_len <1, 0> ]
        [ _start+0.4*_len <1, 0> ]
        [ _start+0.6*_len <1, 0> ]
        [ _start+0.8*_len <1, 0> ]
        [ _start+1.0*_len <1, 0> ]
    #end

    #macro _LowThread(_start, _len)
        [ _start+0.0*_len <0, 0> ]
        [ _start+0.2*_len <0, 0> ]
        [ _start+0.4*_len <0, 0> ]
        [ _start+0.6*_len <0, 0> ]
        [ _start+0.8*_len <0, 0> ]
        [ _start+1.0*_len <0, 0> ]
    #end

    #macro _HighGap(_start, _len)
        [ _start+0.0*_len <1, 0> ]
        [ _start+0.2*_len <1, 0> ]
        [ _start+0.4*_len <1, 0> ]
        [ _start+0.6*_len <1, 0> ]
        [ _start+0.8*_len <1, 0> ]
        [ _start+1.0*_len <1, 0> ]
    #end

    #macro _LowGap(_start, _len)
        [ _start+0.0*_len <0, 0> ]
        [ _start+0.2*_len <0, 0> ]
        [ _start+0.4*_len <0, 0> ]
        [ _start+0.6*_len <0, 0> ]
        [ _start+0.8*_len <0, 0> ]
        [ _start+1.0*_len <0, 0> ]
    #end

    #macro _HighToLowGap(_start, _len)
        [ _start+0.0*_len <1.0, -1> ]
        [ _start+0.2*_len <0.8, -1> ]
        [ _start+0.4*_len <0.6, -1> ]
        [ _start+0.6*_len <0.4, -1> ]
        [ _start+0.8*_len <0.2, -1> ]
        [ _start+1.0*_len <0.0, -1> ]
    #end
    
    #macro _LowToHighGap(_start, _len)
        [ _start+0.0*_len <0.0, 1> ]
        [ _start+0.2*_len <0.2, 1> ]
        [ _start+0.4*_len <0.4, 1> ]
        [ _start+0.6*_len <0.6, 1> ]
        [ _start+0.8*_len <0.8, 1> ]
        [ _start+1.0*_len <1.0, 1> ]
    #end


    /*
     * Create a set of normals according to threads running along X axis
     */

    #local i = 0;

    #local TempNormalX = array[nY];

    #while(i < nY)
        
        #local TempNormalX[i]=
        normal
        {
            gradient x
            slope_map
            {
        
                #local j = 0;
                #local start = 0;

                #while(j < nX)
         
                    #local cur  = ClothWeaveArray[i][j];
                    
                    #if(j = nX-1)
                        #local next = ClothWeaveArray[i][0];
                    #else
                        #local next = ClothWeaveArray[i][j+1];
                    #end
                    
                    #if(cur = 0)
                        #if(next = 0)
                            _HighThread(start, dX)
                            _HighGap(start+dX, dGX)
                        #else
                            _HighThread(start, dX)
                            _HighToLowGap(start+dX, dGX)
                        #end
                    #else
                        #if(next = 0)
                            _LowThread(start, dX)
                            _LowToHighGap(start+dX, dGX)
                        #else
                            _LowThread(start, dX)
                            _LowGap(start+dX, dGX)
                        #end
                    #end
                        

                    #local start = start + dX + dGX;
                                    
                    #local j = j + 1;
                 
                #end

            }
        }
        
        #local i = i + 1;

    #end


    /*
     * Create a set of normals according to threads running along Y axis
     */
    
    #local j = 0;

    #local TempNormalY = array[nX];

    #while(j < nX)
        
        #local TempNormalY[j]=
        normal
        {
            gradient y
            slope_map
            {
        
                #local i = 0;
                #local start = 0;

                #while(i < nY)
         
                    #local cur  = ClothWeaveArray[i][j];
                    
                    #if(i = nY-1)
                        #local next = ClothWeaveArray[0][j];
                    #else
                        #local next = ClothWeaveArray[i+1][j];
                    #end
                    
                    #if(cur = 0)
                        #if(next = 0)
                            _LowThread(start, dY)
                            _LowGap(start+dY, dGY)
                        #else
                            _LowThread(start, dY)
                            _LowToHighGap(start+dY, dGY)
                        #end
                    #else
                        #if(next = 0)
                            _HighThread(start, dY)
                            _HighToLowGap(start+dY, dGY)
                        #else
                            _HighThread(start, dY)
                            _HighGap(start+dY, dGY)
                        #end
                    #end
                        
                    #local start = start + dY + dGY;
                                    
                    #local i = i + 1;
                 
                #end

            }
        }
        
        #local j = j + 1;

    #end


    #local NormalX=
    normal
    {
        gradient y
        
        normal_map
        {
            #local i = 0;
            #local start = 0;
            #local stop = 0;

            #while(i < nY)
                
                #local stop = stop + dY;
                [ start TempNormalX[i]    ]
                [ stop  TempNormalX[i] ]
                #local start = stop;
            
                #local stop = stop + dGY;
                [ start ClearNormal ]
                [ stop  ClearNormal ]
                #local start = stop;
                
                #local i = i + 1;
            #end

        }
    }


    #local NormalY=
    normal
    {
        gradient x
        
        normal_map
        {
            #local j = 0;
            #local start = 0;
            #local stop = 0;

            #while(j < nX)
                
                #local stop = stop + dX;
                [ start TempNormalY[j]    ]
                [ stop  TempNormalY[j] ]
                #local start = stop;
            
                #local stop = stop + dGX;
                [ start ClearNormal ]
                [ stop  ClearNormal ]
                #local start = stop;
                
                #local j = j + 1;
            #end

        }
    }

    average

    normal_map
    {
        [ 1 NormalX ]
        [ 1 NormalY ]
    }

}

// undefine internal macros to free up precious name space
#undef _HighThread
#undef _LowThread
#undef _HighGap
#undef _LowGap
#undef _HighToLowGap
#undef _LowToHighGap

#end  // MakeClothNormal



// vim:textwidth=75:tabstop=4:shiftwidth=4:expandtab
// End of file clothtex.inc
